﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Windows.Forms;

namespace Wallpaper_Resizer {
	public partial class MainInterface : Form {
		
		public MainInterface() {
			InitializeComponent();
			this.Text = Application.ProductName;
		}

		private void MainInterface_Shown(object sender, System.EventArgs e) {
			this.resizerWorker.RunWorkerAsync();
		}

		private void MainInterface_FormClosing(object sender, FormClosingEventArgs e) {
			this.resizerWorker.CancelAsync();
		}

		private void resizerWorker_DoWork(object sender, DoWorkEventArgs e) {
			var sourceDirectory = Directory.GetCurrentDirectory();
			var targetDirectory = Path.GetDirectoryName(sourceDirectory);
			var filesToResize = new List<string>();
			foreach (var extension in new[] { "*.jpg", "*.jpeg", "*.png" }) {
				filesToResize.AddRange(Directory.GetFiles(sourceDirectory, extension));
			}

			var destinationSize = Screen.PrimaryScreen.Bounds;
			var destinationAspectRatio = (float)destinationSize.Width / (float)destinationSize.Height;
			

			var destinationEncoderParameters = new EncoderParameters(1);
			destinationEncoderParameters.Param[0] = new EncoderParameter(Encoder.Quality, (long)95);
			var destinationImageCodecInfo = ImageCodecInfo.GetImageEncoders().First(c => c.MimeType == "image/jpeg");

			for (int fileIndex = 0; !e.Cancel && fileIndex < filesToResize.Count; ++fileIndex) {
				
				var sourceFilename = filesToResize[fileIndex];
				var destinationFilename = Path.Combine(targetDirectory, Path.GetFileNameWithoutExtension(sourceFilename) + ".jpg");

				if (!File.Exists(destinationFilename)) {

					var sourceModifiedTime = File.GetLastWriteTimeUtc(sourceFilename);

					using (var source = new Bitmap(sourceFilename)) {
						using (var destination = new Bitmap(destinationSize.Width, destinationSize.Height)) {
							using (var g = Graphics.FromImage(destination)) {
								g.InterpolationMode = InterpolationMode.HighQualityBicubic;

								// Where are we drawing to?
								var resizedRectangle = destinationSize;

								// Calculate the aspect ratio of the source.
								float sourceAspectRatio = (float)source.Width / (float)source.Height;

								// Handle aspect ratio changes.
								if (sourceAspectRatio > destinationAspectRatio) {
									// Wider to narrower.
									var newWidth = (int)(resizedRectangle.Height * sourceAspectRatio);
									resizedRectangle = new Rectangle((destinationSize.Width - newWidth) / 2, 0, newWidth, resizedRectangle.Height);
								} else if (sourceAspectRatio < destinationAspectRatio) {
									// Taller to shorter.
									var newHeight = (int)(resizedRectangle.Width / sourceAspectRatio);
									resizedRectangle = new Rectangle(0, (destinationSize.Height - newHeight) / 2, resizedRectangle.Width, newHeight);
								}

								// Draw the image.
								g.DrawImage(source, resizedRectangle);
							}

							// Save the result.
							destination.Save(destinationFilename, destinationImageCodecInfo, destinationEncoderParameters);
							File.SetLastWriteTimeUtc(destinationFilename, sourceModifiedTime);
						}
					}
				}
				this.resizerWorker.ReportProgress(100 * fileIndex / filesToResize.Count);
			}
		}

		private void resizerWorker_ProgressChanged(object sender, ProgressChangedEventArgs e) {
			this.conversionProgress.Value = e.ProgressPercentage;
		}

		private void resizerWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
			this.Close();
		}



	}
}
